#
# spec file for package nvml
#
# Copyright (c) 2016 SUSE LINUX GmbH, Nuernberg, Germany.
# Copyright 2016, Intel Corporation
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


Name:           nvml
Version:        1.2.1
Release:        1.1
Summary:        Non-Volatile Memory Library
License:        BSD-3-Clause
Group:          Development/Libraries/C and C++
Url:            http://pmem.io/nvml/

#Git-Clone:	git://github.com/pmem/nvml
Source:         https://github.com/pmem/%{name}/archive/%{version}.tar.gz#/%{name}-%{version}.tar.gz
BuildRequires:  autoconf
BuildRequires:  automake
BuildRequires:  man
BuildRequires:  glibc-devel
BuildRequires:  pkg-config
BuildRequires:  doxygen
Patch0:         build-fix.patch

BuildRoot:      %{_tmppath}/%{name}-%{version}-build

# By design, NVML does not support any 32-bit architecture.
# Due to dependency on xmmintrin.h and some inline assembly, it can be
# compiled only for x86_64 at the moment.
# Other 64-bit architectures could also be supported, if only there is
# a request for that, and if somebody provides the arch-specific
# implementation of the low-level routines for flushing to persistent
# memory.
ExclusiveArch:  x86_64

%description
The NVM Library is a collection of libraries for using memory-mapped
persistence, optimized specifically for persistent memory.

%package tools
Summary:        Utilities for Persistent Memory
Group:          System/Base

%description tools
Useful applications for administration and diagnosis of persistent memory.

%package -n libpmem1
Summary:        Low-level persistent memory support library
Group:          System/Libraries
Recommends:     %name

%description -n libpmem1
libpmem provides low level persistent memory support, in particular,
support for the persistent memory instructions for flushing changes
to pmem.

%package -n libpmem-devel
Summary:        Development files for the low-level persistent memory library
Group:          Development/Libraries/C and C++
Requires:       libpmem1 = %version

%description -n libpmem-devel
libpmem provides low level persistent memory support. In particular,
support for the persistent memory instructions for flushing changes
to pmem is provided.

This library is provided for software which tracks every store to
pmem and needs to flush those changes to durability. Most developers
will find higher level libraries like libpmemobj to be much more
convenient.

%package -n libpmemblk1
Summary:        Persistent Memory Resident Block library
Group:          System/Libraries

%description -n libpmemblk1
libpmemblk implements a pmem-resident array of blocks, all the same
size, where a block is updated atomically with respect to power
failure or program interruption (no torn blocks).

%package -n libpmemblk-devel
Summary:        Development files for the Persistent Memory Resident Block library
Group:          Development/Libraries/C and C++
Requires:       libpmemblk1 = %version

%description -n libpmemblk-devel
libpmemblk implements a pmem-resident array of blocks, all the same
size, where a block is updated atomically with respect to power
failure or program interruption (no torn blocks).

For example, a program keeping a cache of fixed-size objects in pmem
might find this library useful. This library is provided for cases
requiring large arrays of objects at least 512 bytes each. Most
developers will find higher level libraries like libpmemobj to be
more generally useful.

%package -n libpmemlog1
Summary:        Persistent Memory Resident Log File library
Group:          System/Libraries

%description -n libpmemlog1
The libpmemlog library provides a pmem-resident log file. This is
useful for programs like databases that append frequently to a log
file.

%package -n libpmemlog-devel
Summary:        Development files for the Persistent Memory Resident Log File library
Group:          Development/Libraries/C and C++
Requires:       libpmemlog1 = %version

%description -n libpmemlog-devel
The libpmemlog library provides a pmem-resident log file. This
library is provided for cases requiring an append-mostly file to
record variable length entries. Most developers will find higher
level libraries like libpmemobj to be more generally useful.

%package -n libpmemobj1
Summary:        Persistent Memory Transactional Object Store library
Group:          System/Libraries

%description -n libpmemobj1
The libpmemobj library provides a transactional object store,
providing memory allocation, transactions, and general facilities for
persistent memory programming.

%package -n libpmemobj-devel
Summary:        Development files for the Persistent Memory Transactional Object Store library
Group:          Development/Libraries/C and C++
Requires:       libpmemobj1 = %version

%description -n libpmemobj-devel
The libpmemobj library provides a transactional object store,
providing memory allocation, transactions, and general facilities for
persistent memory programming. Developers new to persistent memory
probably want to start with this library.

%package -n libpmemobj++-devel
Summary: C++ bindings for libpmemobj
Group: Development/Libraries
Requires: libpmemobj-devel = %{version}

%description -n libpmemobj++-devel
Development files for NVML C++ libpmemobj bindings

%package -n libvmem1
Summary:        Volatile Memory Pool library
Group:          System/Libraries

%description -n libvmem1
The libvmem library turns a pool of persistent memory into a volatile
memory pool, similar to the system heap but kept separate and with
its own malloc-style API.

%package -n libvmem-devel
Summary:        Development files for the Volatile Memory library
Group:          Development/Libraries/C and C++
Requires:       libvmem1 = %version

%description -n libvmem-devel
The libvmem library turns a pool of persistent memory into a volatile
memory pool, similar to the system heap but kept separate and with
its own malloc-style API.

This subpackage contains libraries and header files for developing
applications that want to make use of libvmem.

%package -n libvmmalloc1
Summary:        Dynamic to Persistent Memory allocation translation library
Group:          System/Libraries

%description -n libvmmalloc1
The libvmmalloc library transparently converts all the dynamic memory
allocations into persistent memory allocations. This allows the use
of persistent memory as volatile memory without modifying the target
application.

The typical usage of libvmmalloc is to load it via the LD_PRELOAD
environment variable.

%package -n libvmmalloc-devel
Summary:        Development files for the Dynamic-to-Persistent allocation library
Group:          Development/Libraries/C and C++
Requires:       libvmmalloc1 = %version

%description -n libvmmalloc-devel
The libvmmalloc library transparently converts all the dynamic memory
allocations into persistent memory allocations. This allows the use
of persistent memory as volatile memory without modifying the target
application.

This subpackage contains libraries and header files for developing
applications that want to specifically make use of libvmmalloc.

%package -n libpmempool1
Summary:        Persistent Memory pool management library
Group:          System/Libraries

%description -n libpmempool1
The libpmempool library provides a set of utilities for off-line administration,
analysis, diagnostics and repair of persistent memory pools created
by libpmemlog, libpemblk and libpmemobj libraries.

%package -n libpmempool-devel
Summary:        Development files for Persistent Memory pool management library
Group:          Development/Libraries/C and C++
Requires:       libpmempool1 = %version

%description -n libpmempool-devel
The libpmempool library provides a set of utilities for off-line administration,
analysis, diagnostics and repair of persistent memory pools created
by libpmemlog, libpemblk and libpmemobj libraries.

%prep
%setup -q
%patch0 -p1

%build
# Currently, NVML makefiles do not allow to easily override CFLAGS,
# so the build flags are passed via EXTRA_CFLAGS.  For debug build
# selected flags are overriden to disable compiler optimizations.
#
# remaining issues:
# * jemalloc attempts to use __builtin_clz, this might not always work
EXTRA_CFLAGS_RELEASE="%{optflags}" \
EXTRA_CFLAGS_DEBUG="%{optflags} -Wp,-U_FORTIFY_SOURCE -O0" \
EXTRA_CXXFLAGS="%{optflags}" \
make %{?_smp_mflags} BINDIR="%_bindir"

# Override LIB_AR with empty string to skip installation of static libraries
%install
b="%buildroot"
make install DESTDIR="$b" LIB_AR= \
	prefix="%_prefix"\
	libdir="%{_libdir}"\
	includedir="%{_includedir}"\
	mandir="%{_mandir}"\
	bindir="%{_bindir}"\
	sysconfdir="%{_sysconfdir}"\
	docdir="%{_docdir}"
mkdir -p "$b/%_datadir/nvml"
cp utils/nvml.magic "$b/%_datadir/nvml/"

%check
cp src/test/testconfig.sh.example src/test/testconfig.sh
#make check

%post   -n libpmem1 -p /sbin/ldconfig
%postun -n libpmem1 -p /sbin/ldconfig
%post   -n libpmemblk1 -p /sbin/ldconfig
%postun -n libpmemblk1 -p /sbin/ldconfig
%post   -n libpmemlog1 -p /sbin/ldconfig
%postun -n libpmemlog1 -p /sbin/ldconfig
%post   -n libpmemobj1 -p /sbin/ldconfig
%postun -n libpmemobj1 -p /sbin/ldconfig
%post   -n libvmem1 -p /sbin/ldconfig
%postun -n libvmem1 -p /sbin/ldconfig
%post   -n libvmmalloc1 -p /sbin/ldconfig
%postun -n libvmmalloc1 -p /sbin/ldconfig
%post   -n libpmempool1 -p /sbin/ldconfig
%postun -n libpmempool1 -p /sbin/ldconfig

%files
%defattr(-,root,root)
%_datadir/nvml/

%files tools
%defattr(-,root,root)
%config %_sysconfdir/bash_completion.d/pmempool.sh
%_bindir/pmempool
%doc LICENSE

%files -n libpmem1
%defattr(-,root,root)
%_libdir/libpmem.so.1*

%files -n libpmem-devel
%defattr(-,root,root)
%_libdir/libpmem.so
%_libdir/pkgconfig/libpmem.pc
%dir %_libdir/nvml_debug/
%_libdir/nvml_debug/libpmem.so*
%_includedir/libpmem.h
%_mandir/man3/libpmem.3*

%files -n libpmemblk1
%defattr(-,root,root)
%_libdir/libpmemblk.so.1*

%files -n libpmemblk-devel
%defattr(-,root,root)
%_libdir/libpmemblk.so
%_libdir/pkgconfig/libpmemblk.pc
%dir %_libdir/nvml_debug/
%_libdir/nvml_debug/libpmemblk.so*
%_includedir/libpmemblk.h
%_mandir/man3/libpmemblk.3*

%files -n libpmemlog1
%defattr(-,root,root)
%_libdir/libpmemlog.so.1*

%files -n libpmemlog-devel
%defattr(-,root,root)
%_libdir/libpmemlog.so
%_libdir/pkgconfig/libpmemlog.pc
%dir %_libdir/nvml_debug/
%_libdir/nvml_debug/libpmemlog.so*
%_includedir/libpmemlog.h
%_mandir/man3/libpmemlog.3*

%files -n libpmemobj1
%defattr(-,root,root)
%_libdir/libpmemobj.so.1*

%files -n libpmemobj-devel
%defattr(-,root,root)
%_libdir/libpmemobj.so
%dir %_libdir/nvml_debug/
%_libdir/pkgconfig/libpmemobj.pc
%_libdir/nvml_debug/libpmemobj.so*
%_includedir/libpmemobj.h
%dir %_includedir/libpmemobj
%_includedir/libpmemobj/*.h
%_mandir/man3/libpmemobj.3*

%files -n libvmem1
%defattr(-,root,root)
%_libdir/libvmem.so.1*

%files -n libvmem-devel
%defattr(-,root,root)
%_libdir/libvmem.so
%_libdir/pkgconfig/libvmem.pc
%dir %_libdir/nvml_debug/
%_libdir/nvml_debug/libvmem.so*
%_includedir/libvmem.h
%_mandir/man3/libvmem.3*

%files -n libvmmalloc1
%defattr(-,root,root)
%_libdir/libvmmalloc.so.1*

%files -n libvmmalloc-devel
%defattr(-,root,root)
%_libdir/libvmmalloc.so
%_libdir/pkgconfig/libvmmalloc.pc
%dir %_libdir/nvml_debug/
%_libdir/nvml_debug/libvmmalloc.so*
%_includedir/libvmmalloc.h
%_mandir/man3/libvmmalloc.3*

%files -n libpmempool1
%defattr(-,root,root)
%_libdir/libpmempool.so.1*
%_mandir/man1/pmempool-*.1*
%_mandir/man1/pmempool.1*

%files -n libpmempool-devel
%defattr(-,root,root)
%_libdir/libpmempool.so
%_libdir/pkgconfig/libpmempool.pc
%dir %_libdir/nvml_debug
%_libdir/nvml_debug/libpmempool.so*
%_includedir/libpmempool.h
%_mandir/man3/libpmempool.3*

%files -n libpmemobj++-devel
%defattr(-,root,root,-)
%{_libdir}/pkgconfig/libpmemobj++.pc
%dir %{_includedir}/libpmemobj++/
%{_includedir}/libpmemobj++/*.hpp
%dir %{_includedir}/libpmemobj++/detail
%{_includedir}/libpmemobj++/detail/*.hpp
%dir %{_docdir}/libpmemobj++-devel/
%{_docdir}/libpmemobj++-devel/*

%changelog
* Thu Mar 16 2017 jthumshirn@suse.com
- Update to upstream version 1.2.1 (FATE#321727)
  This NVML release changes the behavior of pmem_is_pmem() on Linux.
  The pmem_is_pmem() function will now return true only if the entire
  range is mapped directly from Device DAX (/dev/daxX.Y) without an
  intervening file system, and only if the corresponding file mapping
  was created with pmem_map_file().  See libpmem(3) for details.
* Thu Mar  9 2017 jthumshirn@suse.com
- Update to upstream version 1.2 (FATE#321727)
  + build-fix.patch
- 0001-libpmemblk-cast-translation-entry-to-uint64_t-when-calculating-data-offset.patch
  * Add Device DAX support, providing that "optimized flush" mechanism
  defined in SNIA NVM Programming Model can safely be used, even
  if PMEM-aware file system supporting that model is not available,
  or if the user does not want to use the file system for some reason.
  * Add a package for libpmemobj C++ bindings.
  C++ API is no longer considered experimental.
  Web-based documentation for C++ API is available on http://pmem.io.
  * Add "sync" and "transform" commands to pmempool utility.
  The "sync" command allows to recover missing or corrupted part(s)
  of a pool set from a healthy replica, while the "transform" command
  is a convenient way for modifying the structure of an existing
  pool set, i.e. by adding or removing replicas.
  * Add experimental support for remote access to persistent memory and
  basic remote data replication over RDMA (librpmem).  Experimental
  support for remote replicas is also provided by libpmemobj library.
  Note: This is one not packaged for SUSE yet.
* Thu Jul  7 2016 jthumshirn@suse.com
- Fix 32 Bit overflow in resulting in flying writes (bsc#986657)
  + 0001-libpmemblk-cast-translation-entry-to-uint64_t-when-calculating-data-offset.patch
* Sun Jun 26 2016 krzysztof.czurylo@intel.com
- Update to upstream version 1.1 (FATE#319673)
  * A new version of libpmemobj pool layout is introduced.
  * Internal undo log structure has been modified to improve
  performance of pmemobj transactions.
  * Memory pools created with older versions of the libpmemobj
  library must be converted to the new format using "pmempool
  convert" command. See pmempool-convert(1) for details.
  * A new "libpmempool" library is available, providing support for
  off-line pool management and diagnostics. Initially, it
  provides only "check" and "repair" operations for log and blk
  memory pools, and for BTT devices.
  * pmem: deprecate PCOMMIT
  * blk: match BTT Flog initialization with Linux NVDIMM BTT
  * pmem: defer pmem_is_pmem() initialization
  * cpp: clarify polymorphism in persistent_ptr
  * obj: let the "before" flag be any non-zero value
  * obj: fix compare array pptr to nullptr
  * log/blk: set errno if replica section is specified
- Remove libuuid dependency
- Update link to source tarball (x.y.tar.gz => nvml-x.y.tar.gz)
- Add libpmempool subpackage
- Add new BuildRequires: pkg-config, glibc-devel
* Thu Apr  7 2016 jthumshirn@suse.com
- Version 1.0
    The API of six libraries (libpmem, libpmemblk, libpmemlog,
    libpmemobj, libvmem, libvmmalloc) is complete and stable.
    The on-media layout of persistent memory pools will be maintained
    from this point, and if changed it will be backward compatible.
    Man pages are all complete.
    This release has been validated to "Production quality".
    For the purpose of new features planned for next releases of NVML
    there have been some API modifications made:
  - pmem: pmem_map replaced with pmem_map_file
  - log/blk: 'off_t' substituted with 'long long'
  - obj: type numbers extended to 64-bit
  - obj: new entry points and macros added:
    pmemobj_tx_errno, pmemobj_tx_lock, pmemobj_mutex_timedlock,
    TX_ADD_DIRECT, TX_ADD_FIELD_DIRECT, TX_SET_DIRECT
    Other key changes since version 0.4 include:
  - common: updated/fixed installation scripts
  - common: eliminated dependency on libuuid
  - pmem: CPU features/ISA detection using CPUID
  - obj: improved error handling
  - obj: atomic allocation fails if constructor returns error
  - obj: multiple performance optimizations
  - obj: object store refactoring
  - obj: additional examples and benchmarks
    This release also introduces a prototype implementation
    of C++ bindings for libpmemobj.  Note that C++ API is still
    experimental and should not be used in production environments.
* Fri Jan 22 2016 jthumshirn@suse.com
- Update to upstream version 0.4
  This NVML version primarily focuses on improving code quality
  and reliability.  In addition to a couple of bug fixes,
  the changes include:
  - benchmarks for libpmemobj, libpmemblk and libvmem
  - additional pmemobj tests and examples
  - pool mapping address randomization
  - added pmempool "rm" command
  - eliminated libpmem dependency on libpthread
  - enabled extra warnings
  - minor performance improvements
  Man pages are all complete.
  This release is considered "Beta quality" by the team, having
  been thoroughly validated, including significant performance analysis.
  The pmempool command does not yet support "check" and "repair"
  operations for pmemobj type pools.
* Mon Oct 19 2015 jthumshirn@suse.com
- Update to upstream version 0.3
  NVML is now feature complete, adding support for:
  - pool sets
  - pmemobj local replication (active/passive)
  - experimental valgrind support
  - pmempool support for all pool types
  Man pages are all complete.
  This release is considered "Alpha quality" by the team, having
  gone through significant validation but only some performance
  analysis at this point.
* Fri Jul 10 2015 jthumshirn@suse.com
- Initial packaging for SUSE (version 0.2)
